"""
This type stub file was generated by pyright.
"""

from pactman.mock.matchers import EachLike, Equals, Includes, Like, SomethingLike, Term
from pactman.mock.provider import Provider

from .mock.consumer import Consumer
from .mock.pact import Pact

__all__ = ['Consumer', 'Pact', 'Provider', 'EachLike', 'Equals', 'Includes', 'Like', 'SomethingLike', 'Term']
"""Python methods for interactive with a Pact Mock Service."""
