"""
This type stub file was generated by pyright.
"""

import logging
from typing import NoReturn

log = logging.getLogger(__name__)

class Result:
    PASS: bool = ...
    FAIL: bool = ...
    success: bool = ...
    def start(self, message: str) -> None: ...
    def end(self) -> None: ...
    def warn(self, message: str) -> NoReturn: ...
    def fail(self, message: str, path: str = ...) -> NoReturn: ...
