"""
This type stub file was generated by pyright.
"""
from typing import Dict, List, NoReturn

from pactman import Pact
from pactman.mock.pact import Interaction
from urllib3.response import HTTPResponse

from .pact_request_handler import PactRequestHandler

class MockURLOpenHandler(PactRequestHandler):
    path: str
    pact: Pact
    headers: Dict[str, str]
    interactions: List[Interaction]
    def __init__(self, config: Pact) -> None: ...
    def terminate(self) -> None: ...
    def setup(self, interactions: List[Interaction]) -> None: ...
    def verify(self) -> None: ...
    def __call__(
        self, method: str, url: str, redirect: bool = ..., headers: Dict[str, str] = ..., body: bytes = ..., **kw: object
    ) -> HTTPResponse: ...
    def get_interaction(self, path: str) -> Interaction: ...
    def handle_failure(self, reason: str) -> NoReturn: ...
    def handle_success(self, interaction: Interaction) -> None: ...
    def respond_for_interaction(self, interaction: Interaction) -> HTTPResponse: ...
